@@ -1,16 +1,20 @@
 
 import os
+import sys
 import time
 import yaml
-
+import datetime
 from typeguard import typechecked
+from tenacity import retry, stop_after_attempt
+
 from benchmark_runner.common.logger.logger_time_stamp import logger_time_stamp, logger
 from benchmark_runner.common.elasticsearch.elasticsearch_exceptions import ElasticSearchDataNotUploaded
 from benchmark_runner.common.oc.oc import OC
 from benchmark_runner.common.oc.oc_exceptions import VMNotCompletedTimeout
-from benchmark_runner.benchmark_operator.templates.generate_yaml_from_templates import TemplateOperations
+from benchmark_runner.benchmark_operator.workload_flavors.generate_yaml_from_workload_flavors import TemplateOperations
 from benchmark_runner.common.elasticsearch.es_operations import ESOperations
 from benchmark_runner.common.ssh.ssh import SSH
+from benchmark_runner.benchmark_operator.benchmark_operator_exceptions import OCSNonInstalled, SystemMetricsRequiredElasticSearch
 from benchmark_runner.main.environment_variables import environment_variables
 
 
@@ -19,25 +23,28 @@ class BenchmarkOperatorWorkloads:
     This class contains all the custom_workloads
     """
     def __init__(self, kubeadmin_password: str = '', es_host: str = '', es_port: str = ''):
+        # environment variables
+        self.__environment_variables_dict = environment_variables.environment_variables_dict
+        self.__runner_version = self.__environment_variables_dict.get('build_version', '')
+        self.__run_type = self.__environment_variables_dict.get('run_type', '')
+        self.__stop_when_workload_finish = self.__environment_variables_dict.get('stop_when_workload_finish', '')
+        self.__system_metrics = self.__environment_variables_dict.get('system_metrics', '')
+        self.__elasticsearch = self.__environment_variables_dict.get('elasticsearch', '')
         self.__ssh = SSH()
         self.__kubeadmin_password = kubeadmin_password
         self.__oc = OC(kubeadmin_password=self.__kubeadmin_password)
-        #  if mount is exist - path inside Dockerfile
-        if os.path.exists('/benchmark_runner/templates/'):
-            self.__dir_path = '/benchmark_runner/templates'
-            self.__current_run_path = f'{self.__dir_path}/current_run'
-        else:
-            self.__dir_path = os.path.dirname(os.path.realpath(__file__))
-            self.__current_run_path = f'{self.__dir_path}/templates/current_run'
+        self.__dir_path = f'{os.path.dirname(os.path.realpath(__file__))}'
+        self.__current_run_path = f'{self.__dir_path}/workload_flavors/{self.__run_type}/current_run'
         self.__es_host = es_host
         self.__es_port = es_port
         if es_host and es_port:
             self.__es_operations = ESOperations(es_host=self.__es_host, es_port=self.__es_port)
-        # Generate templates class
+        # Generate workload_flavors class
         self.__template = TemplateOperations()
-        self.__oc.login()
-        # environment variables
-        self.__environment_variables_dict = environment_variables.environment_variables_dict
+
+        # Login when kubeadmin_password
+        if self.__environment_variables_dict.get('kubeadmin_password', ''):
+            self.__oc.login()
 
     def __remove_current_run_yamls(self, extension='.yaml'):
         """
@@ -64,6 +71,25 @@ class BenchmarkOperatorWorkloads:
         else:
             logger.info('yaml file {} does not exist')
 
+    def __exist_for_debug(self):
+        """
+        This method check for workload exist when complete running
+        :return:
+        """
+        # Stop for debugging workload pod
+        if self.__stop_when_workload_finish == 'True':
+            return sys.exit(0)
+
+    def __check_elasticsearch_exist_for_system_metrics(self):
+        """
+        This method check if elasticsearch exist for system metrics
+        :return:
+        """
+        # Stop for debugging workload pod
+        if self.__system_metrics == 'True':
+            if not self.__elasticsearch:
+                raise SystemMetricsRequiredElasticSearch()
+
     @logger_time_stamp
     def update_node_selector(self, runner_path: str = environment_variables.environment_variables_dict['runner_path'], yaml_path: str = '', pin_node: str = ''):
         """
@@ -156,7 +182,7 @@ class BenchmarkOperatorWorkloads:
         os.chdir(os.path.join(runner_path, benchmark_operator_path))
         self.__ssh.run('/usr/local/bin/helm install benchmark-operator . -n benchmark-operator --create-namespace')
         self.__oc.wait_for_pod_create(pod_name='benchmark-operator')
-        self.__ssh.run('~/./oc adm policy -n benchmark-operator add-scc-to-user privileged -z benchmark-operator')
+        self.__ssh.run('oc adm policy -n benchmark-operator add-scc-to-user privileged -z benchmark-operator')
         os.chdir(current_dir)
 
     @logger_time_stamp
@@ -221,14 +247,62 @@ class BenchmarkOperatorWorkloads:
         self.__oc.wait_for_pod_completed(label='app=system-metrics-collector', workload=workload)
         # verify that data upload to elastic search
         if self.__es_host:
-            self.__es_operations.verify_es_data_uploaded(index=f'system-metrics',
-                                                         uuid=self.__oc.get_long_uuid(workload=workload))
+            if self.__run_type == 'test_ci':
+                es_index = 'system-metrics-test-ci'
+            else:
+                es_index = 'system-metrics'
+            self.__es_operations.verify_es_data_uploaded(index=es_index,
+                                                         uuid=self.__oc.get_long_uuid(workload=workload), fast_check=True)
+
+    def get_metadata(self, kind: str = None, database: str = None):
+        """
+        This method return metadata kind and database argument are optional
+        @param kind: optional: pod or vm
+        @param database: optional:mssql, postgres or mariadb
+        :return:
+        """
+        date_format = '%Y_%m_%d'
+        metadata = {'ocp_version': self.__oc.get_ocp_server_version(),
+                    'cnv_version': self.__oc.get_cnv_version(),
+                    'ocs_version': self.__oc.get_ocs_version(),
+                    'runner_version': self.__runner_version,
+                    'version': int(self.__runner_version.split('.')[-1]),
+                    'ci_date': datetime.datetime.now().strftime(date_format)}
+        if kind:
+            metadata.update({'kata_version': '',
+                             'kind': kind,
+                             'vm_os_version': 'centos8'})
+        # for hammerdb
+        if database == 'mssql':
+            metadata.update({'db_version': 2019})
+        elif database == 'postgres':
+            metadata.update({'db_version': 10})
+        elif database == 'mariadb':
+            metadata.update({'db_version': 10.3})
+        return metadata
+
+    def update_ci_status(self, status: str, ci_minutes_time: int, benchmark_operator_id: str, benchmark_wrapper_id: str, ocp_install_minutes_time: int = 0, ocp_resource_install_minutes_time: int = 0):
+        """
+        This method update ci status Pass/Failed
+        :param status: Pass/Failed
+        :param ci_minutes_time: ci time in minutes
+        :param benchmark_wrapper_id: benchmark_wrapper last repository commit id
+        :param benchmark_operator_id: benchmark_operator last repository commit id
+        :param ocp_install_minutes_time: ocp install minutes time, default 0 because ocp install run once a week
+        :param ocp_resource_install_minutes_time: ocp install minutes time, default 0 because ocp install run once a week
+        :return:
+        """
+        status_dict = {'failed': 0, 'pass': 1}
+        metadata = self.get_metadata()
+        metadata.update({'status': status, 'status#': status_dict[status], 'ci_minutes_time': ci_minutes_time, 'benchmark_operator_id': benchmark_operator_id, 'benchmark_wrapper_id': benchmark_wrapper_id, 'ocp_install_minutes_time': ocp_install_minutes_time, 'ocp_resource_install_minutes_time': ocp_resource_install_minutes_time})
+        self.__es_operations.upload_to_es(index='ci-status', data=metadata)
 
 #***********************************************************************************************
 ######################################## Workloads #############################################
 #***********************************************************************************************
 
     @logger_time_stamp
+    @retry(stop=stop_after_attempt(3))
     def stressng_pod(self):
         """
         This method run stressng workload
@@ -240,12 +314,20 @@ class BenchmarkOperatorWorkloads:
             self.__oc.wait_for_initialized(label='app=stressng_workload', workload=workload)
             self.__oc.wait_for_ready(label='app=stressng_workload', workload=workload)
             self.__oc.wait_for_pod_completed(label='app=stressng_workload', workload=workload)
+            self.__exist_for_debug()
             # system metrics
             if environment_variables.environment_variables_dict['system_metrics'] == 'True':
                 self.system_metrics_collector(workload=workload)
             if self.__es_host:
+                if self.__run_type == 'test_ci':
+                    es_index = 'stressng-test-ci-results'
+                else:
+                    es_index = 'stressng-results'
                 # verify that data upload to elastic search according to unique uuid
-                self.__es_operations.verify_es_data_uploaded(index=f'{workload}-results', uuid=self.__oc.get_long_uuid(workload=workload))
+                ids = self.__es_operations.verify_es_data_uploaded(index=es_index, uuid=self.__oc.get_long_uuid(workload=workload))
+                # update metadata
+                for id in ids:
+                    self.__es_operations.update_es_index(index=es_index, id=id, metadata=self.get_metadata(kind='pod'))
             self.__oc.delete_pod_sync(
                 yaml=os.path.join(f'{self.__current_run_path}', f'{self.stressng_pod.__name__}.yaml'),
                 pod_name=f'{workload}-workload')
@@ -259,6 +341,7 @@ class BenchmarkOperatorWorkloads:
             raise err
 
     @logger_time_stamp
+    @retry(stop=stop_after_attempt(3))
     def stressng_vm(self):
         """
         This method run stressng vm workload
@@ -270,12 +353,20 @@ class BenchmarkOperatorWorkloads:
             self.__oc.wait_for_initialized(label='app=stressng_workload', workload=workload)
             self.__oc.wait_for_ready(label='app=stressng_workload', workload=workload)
             self.__oc.wait_for_vm_completed(workload=workload)
+            self.__exist_for_debug()
             # system metrics
             if environment_variables.environment_variables_dict['system_metrics'] == 'True':
                self.system_metrics_collector(workload=workload)
             # verify that data upload to elastic search
             if self.__es_host:
-                self.__es_operations.verify_es_data_uploaded(index=f'{workload}-results', uuid=self.__oc.get_long_uuid(workload=workload))
+                if self.__run_type == 'test_ci':
+                    es_index = 'stressng-test-ci-results'
+                else:
+                    es_index = 'stressng-results'
+                ids = self.__es_operations.verify_es_data_uploaded(index=es_index, uuid=self.__oc.get_long_uuid(workload=workload))
+                # update metadata
+                for id in ids:
+                    self.__es_operations.update_es_index(index=es_index, id=id, metadata=self.get_metadata(kind='vm'))
             self.__oc.delete_vm_sync(
                 yaml=os.path.join(f'{self.__current_run_path}', f'{self.stressng_vm.__name__}.yaml'),
                 vm_name=f'{workload}-workload')
@@ -289,6 +380,7 @@ class BenchmarkOperatorWorkloads:
             raise err
 
     @logger_time_stamp
+    @retry(stop=stop_after_attempt(3))
     def uperf_pod(self):
         """
         This method run uperf workload
@@ -299,22 +391,37 @@ class BenchmarkOperatorWorkloads:
             self.__oc.create_pod_sync(yaml=os.path.join(f'{self.__current_run_path}', f'{self.uperf_pod.__name__}.yaml'), pod_name=f'uperf-server')
             # uperf server
             server_name = self.__environment_variables_dict.get('pin_node1', '')
-            # name up to 27 chars
-            if len(server_name) > 27:
-                server_name = server_name[:28]
-            self.__oc.wait_for_initialized(label=f'app=uperf-bench-server-{server_name}-0', workload=workload)
-            self.__oc.wait_for_ready(label=f'app=uperf-bench-server-{server_name}-0', workload=workload)
+            if server_name:
+                # uperf server name is limited up to 27 chars
+                if len(server_name) > 27:
+                    server_name = server_name[:28]
+                label = f'app=uperf-bench-server-{server_name}-0'
+                self.__oc.wait_for_initialized(label=label, workload=workload)
+                self.__oc.wait_for_ready(label=label, workload=workload)
+            # in case that no pin node
+            else:
+                label = f'benchmark-operator-workload=uperf'
+                self.__oc.wait_for_initialized(label=label, workload=workload, label_uuid=False)
+                self.__oc.wait_for_ready(label=label, workload=workload, label_uuid=False)
             # uperf client
             self.__oc.wait_for_pod_create(pod_name=f'uperf-client')
             self.__oc.wait_for_initialized(label='app=uperf-bench-client', workload=workload)
             self.__oc.wait_for_ready(label='app=uperf-bench-client', workload=workload)
             self.__oc.wait_for_pod_completed(label='app=uperf-bench-client', workload=workload)
+            self.__exist_for_debug()
             # system metrics
             if environment_variables.environment_variables_dict['system_metrics'] == 'True':
                 self.system_metrics_collector(workload=workload)
             if self.__es_host:
+                if self.__run_type == 'test_ci':
+                    es_index = 'uperf-test-ci-results'
+                else:
+                    es_index = 'uperf-results'
                 # verify that data upload to elastic search
-                self.__es_operations.verify_es_data_uploaded(index=f'{workload}-results', uuid=self.__oc.get_long_uuid(workload=workload), workload=self.uperf_pod.__name__)
+                ids = self.__es_operations.verify_es_data_uploaded(index=es_index, uuid=self.__oc.get_long_uuid(workload=workload), workload=self.uperf_pod.__name__)
+                # update metadata
+                for id in ids:
+                    self.__es_operations.update_es_index(index=es_index, id=id, metadata=self.get_metadata(kind='pod'))
             self.__oc.delete_pod_sync(
                 yaml=os.path.join(f'{self.__current_run_path}', f'{self.uperf_pod.__name__}.yaml'),
                 pod_name=f'uperf-client')
@@ -326,6 +433,7 @@ class BenchmarkOperatorWorkloads:
             raise err
 
     @logger_time_stamp
+    @retry(stop=stop_after_attempt(3))
     def uperf_vm(self):
         """
         This method run uperf vm workload
@@ -343,12 +451,20 @@ class BenchmarkOperatorWorkloads:
             self.__oc.wait_for_initialized(label='app=uperf-bench-client', workload=workload)
             self.__oc.wait_for_ready(label='app=uperf-bench-client', workload=workload)
             self.__oc.wait_for_vm_completed(workload=workload)
+            self.__exist_for_debug()
             # system metrics
             if environment_variables.environment_variables_dict['system_metrics'] == 'True':
                 self.system_metrics_collector(workload=workload)
             # verify that data upload to elastic search
             if self.__es_host:
-                self.__es_operations.verify_es_data_uploaded(index=f'{workload}-results', uuid=self.__oc.get_long_uuid(workload=workload), workload=self.uperf_vm.__name__)
+                if self.__run_type == 'test_ci':
+                    es_index = 'uperf-test-ci-results'
+                else:
+                    es_index = 'uperf-results'
+                ids = self.__es_operations.verify_es_data_uploaded(index=es_index, uuid=self.__oc.get_long_uuid(workload=workload), workload=self.uperf_vm.__name__)
+                # update metadata
+                for id in ids:
+                    self.__es_operations.update_es_index(index=es_index, id=id, metadata=self.get_metadata(kind='vm'))
             self.__oc.delete_vm_sync(yaml=os.path.join(f'{self.__current_run_path}', f'{self.uperf_vm.__name__}.yaml'),
                                      vm_name='uperf-server')
         except ElasticSearchDataNotUploaded as err:
@@ -360,6 +476,7 @@ class BenchmarkOperatorWorkloads:
 
     @typechecked
     @logger_time_stamp
+    @retry(stop=stop_after_attempt(3))
     def hammerdb_pod(self, database: str):
         """
         This method run hammerdb pod workload
@@ -383,12 +500,20 @@ class BenchmarkOperatorWorkloads:
             self.__oc.wait_for_initialized(label='app=hammerdb_workload', workload=workload)
             self.__oc.wait_for_ready(label='app=hammerdb_workload', workload=workload)
             self.__oc.wait_for_pod_completed(label='app=hammerdb_workload', workload=workload)
+            self.__exist_for_debug()
             # system metrics
             if environment_variables.environment_variables_dict['system_metrics'] == 'True':
                 self.system_metrics_collector(workload=workload)
             if self.__es_host:
+                if self.__run_type == 'test_ci':
+                    es_index = 'hammerdb-test-ci-results'
+                else:
+                    es_index = 'hammerdb-results'
                 # verify that data upload to elastic search
-                self.__es_operations.verify_es_data_uploaded(index=f'hammerdb-results', uuid=self.__oc.get_long_uuid(workload=workload))
+                ids = self.__es_operations.verify_es_data_uploaded(index=es_index, uuid=self.__oc.get_long_uuid(workload=workload))
+                # update metadata
+                for id in ids:
+                    self.__es_operations.update_es_index(index=es_index, id=id, metadata=self.get_metadata(kind='pod', database=database))
             # delete hammerdb
             self.__oc.delete_pod_sync(
                 yaml=os.path.join(f'{self.__current_run_path}', f'{self.hammerdb_pod.__name__}_{database}.yaml'),
@@ -416,6 +541,7 @@ class BenchmarkOperatorWorkloads:
 
     @typechecked
     @logger_time_stamp
+    @retry(stop=stop_after_attempt(3))
     def hammerdb_vm(self, database: str):
         """
         This method run hammerdb vm workload
@@ -429,12 +555,20 @@ class BenchmarkOperatorWorkloads:
             self.__oc.wait_for_initialized(label='app=hammerdb_workload', workload=workload)
             self.__oc.wait_for_ready(label='app=hammerdb_workload', workload=workload)
             self.__oc.wait_for_vm_completed(workload=workload)
+            self.__exist_for_debug()
             # system metrics
             if environment_variables.environment_variables_dict['system_metrics'] == 'True':
                 self.system_metrics_collector(workload=workload)
             # verify that data upload to elastic search
             if self.__es_host:
-                self.__es_operations.verify_es_data_uploaded(index=f'hammerdb-results', uuid=self.__oc.get_long_uuid(workload=workload))
+                if self.__run_type == 'test_ci':
+                    es_index = 'hammerdb-test-ci-results'
+                else:
+                    es_index = 'hammerdb-results'
+                ids = self.__es_operations.verify_es_data_uploaded(index=es_index, uuid=self.__oc.get_long_uuid(workload=workload))
+                # update metadata
+                for id in ids:
+                    self.__es_operations.update_es_index(index=es_index, id=id, metadata=self.get_metadata(kind='vm', database=database))
             self.__oc.delete_vm_sync(
                 yaml=os.path.join(f'{self.__current_run_path}', f'{self.hammerdb_vm.__name__}_{database}.yaml'),
                 vm_name=f'{workload}-workload')
@@ -463,6 +597,10 @@ class BenchmarkOperatorWorkloads:
         self.remove_if_exist_run_yaml()
         workload_name = workload_full_name.split('_')
         if 'hammerdb' in workload_full_name:
+            # check if ocs is installed
+            if self.__environment_variables_dict.get('ocs_pvc', '') == 'True':
+                if not self.__oc.is_ocs_installed():
+                    raise OCSNonInstalled()
             self.__template.generate_hammerdb_yamls(workload=f'{workload_name[0]}_{workload_name[1]}', database=workload_name[2])
             class_method = getattr(BenchmarkOperatorWorkloads, f'{workload_name[0]}_{workload_name[1]}')
             class_method(self, workload_name[2])
@@ -480,6 +618,9 @@ class BenchmarkOperatorWorkloads:
         :return:
         """
 
+        # check that there is elasticsearch when system metric is True
+        self.__check_elasticsearch_exist_for_system_metrics()
+
         # make undeploy benchmark controller manager if exist
         self.make_undeploy_benchmark_controller_manager_if_exist(runner_path=environment_variables.environment_variables_dict['runner_path'])
 
