@@ -1,7 +1,8 @@
 
 import os
 from jinja2 import Template
-from benchmark_runner.main.environment_variables import EnvironmentVariables
+from benchmark_runner.main.environment_variables import environment_variables
+from benchmark_runner.common.oc.oc import OC
 
 
 def update_environment_variable(dir_path: str, yaml_file: str, environment_variable_dict: dict = None):
@@ -13,8 +14,10 @@ def update_environment_variable(dir_path: str, yaml_file: str, environment_varia
     :return:
     """
     if not environment_variable_dict:
-        environment_variable = EnvironmentVariables()
-        environment_variable_dict = environment_variable.environment_variables_dict
+        environment_variable_dict = environment_variables.environment_variables_dict
+    oc = OC(kubeadmin_password=environment_variable_dict.get('kubeadmin_password', ''))
+    # set prom token
+    environment_variable_dict['prom_token'] = oc.get_prom_token()
     with open(os.path.join(dir_path, yaml_file)) as f:
         template_str = f.read()
     tm = Template(template_str)
